﻿##### Scriptinfo ##################################################################################
# Scriptreihe:      AD-SecurityScopes Arbeitsfunktionen
# Datum:            2019-01-22
# Version:          V1.13
# Programmierer:    Stephan Walther
###################################################################################################

# Funktionen
    function Protokoll {
        param(
            $Eintrag,
            [validateset('green','white','yellow','magenta','cyan')]$ForegroundColor            
        )

        $Zeile = (Get-Date -Format u) -replace 'Z' -replace ' ','-' -replace ':','-'
        $Zeile = $Zeile + ' ' + $env:USERDOMAIN + '\' + $env:USERNAME
        $Zeile = $Zeile + ' ' + $Eintrag
        
        $Zeile | Out-File -Append -FilePath $LogFilePfad 
        
        Write-Host $Eintrag -ForegroundColor $ForegroundColor
    }

    function teste-ADGroup {
        param($Groupname)
        if( -not (Get-ADGroup -Filter {Samaccountname -eq $Groupname})) {         
            Return $false
        } else {
            Return $true 
        }
    }

    function erstelle-ADGroups {
        param($GroupName,$GroupScope,$GroupPath,$GroupDescription='')

        if( -not (Get-ADGroup -Filter {Samaccountname -eq $GroupName})) { 
            if (Get-ADOrganizationalUnit -Filter "DistinguishedName -eq '$GroupPath'") {
                New-ADGroup -Name $GroupName -GroupScope $GroupScope -path $GroupPath -description $GroupDescription -ErrorAction SilentlyContinue | Out-Null
       
                if (-not $?) {
                    Protokoll -Eintrag ("      Fehler: " + $Error[0].Exception.Message) -ForegroundColor magenta
                } else {
                    Protokoll -Eintrag "      Gruppe $GroupName wurde erstellt" -ForegroundColor green
                }
            } else {
                Protokoll -Eintrag ("      Fehler: " + $Error[0].Exception.Message) -ForegroundColor magenta
            }        
        } else {
            Protokoll -Eintrag "      Gruppe $GroupName gibt es schon" -ForegroundColor yellow
        }
    }

    function neues-ADGroupMember {
        param($GroupName,$MemberName)

        $g = $GroupName  #Get-ADGroup -Identity $GroupName | Select-Object -ExpandProperty name
        $m = $MemberName #Get-ADObject -Identity $MemberName | Select-Object -ExpandProperty name

        if (teste-ADGroup -Groupname $GroupName) {
            if( (Get-ADGroupMember $GroupName | Where-Object {$_.samaccountname -eq $MemberName}) -eq $Null        ) { 
                Protokoll -Eintrag "      Gruppenmitglied $m -> $g wird erstellt" -ForegroundColor green
                Add-ADGroupMember $GroupName -Members $MemberName -ErrorAction SilentlyContinue | Out-Null 

                if (-not $?) {
                    Protokoll -Eintrag ("      Fehler: " + $Error[0].Exception.Message) -ForegroundColor magenta
                }
            } else {
                Protokoll -Eintrag "      Gruppenmitglied $m -> $g ist bereits vorhanden" -ForegroundColor yellow
            }
        } else {
            Protokoll -Eintrag "      FEHLER: Gruppe $GroupName wurde nicht gefunden!" -ForegroundColor magenta
        }
    }

    function ermittle-SID {
        param ($ADObject)

        (Get-ADObject -Filter "samaccountname -eq '$ADObject'" -Properties objectSid).objectSid.value
    }

    function erstelle_OUs {
        param($OUName,$OUPfad='',$OUDescription,$ProtOffset="      ")

        if ($OUPfad -eq '') {
            $Pfad = "$OUName"
            $OU_Name = (($OUName -split ',')[0] -split '=')[1]
            $OU_Pfad = ($OUName -split ',' | Select-Object -Skip 1) -join ','
        } else {
            $Pfad = "OU=$OUName,$OUPfad"
            $OU_Name = $OUName
            $OU_Pfad = $OUPfad
        }        
      
        if( -not (Get-ADOrganizationalUnit -Filter {DistinguishedName -eq $Pfad})) { 
            Protokoll -Eintrag "$ProtOffset$Pfad" -ForegroundColor green
            New-ADOrganizationalUnit -name $OU_Name -path $OU_Pfad -ProtectedFromAccidentalDeletion $TRUE -Description $OUDescription -ErrorAction SilentlyContinue | Out-Null
        
            if (-not $?) {
                Protokoll -Eintrag ("$ProtOffset" + "Fehler: " + $Error[0].Exception.Message) -ForegroundColor magenta
            }
        } else {
            Protokoll -Eintrag "$ProtOffset$Pfad gibt es schon" -ForegroundColor yellow
        }
    }

    function erstelle-ADUser {
        param($UserName,$UserPath,$UserDescription="")

        if( -not (Get-ADUser -Filter "SamAccountName -eq '$UserName'")) {      
            $PWD = Read-Host -AsSecureString -Prompt "   Passwort von $UserName"
            New-ADUser `
                -Name $UserName `
                -displayname $UserName `
                -samaccountname $UserName `
                -UserPrincipalName "$UserName@$((Get-ADDomain).DNSRoot)" `
                -Path $UserPath `
                -Enabled 1 `
                –ChangePasswordAtLogon 0 `
                -AccountPassword $PWD `
                -Description $UserDescription
            Protokoll -Eintrag "   erstelle $UserName" -ForegroundColor green
        } else {
            Protokoll -Eintrag "   $UserName existiert bereits" -ForegroundColor yellow
        }
    }

    function erstelle-AdminArea {
        param(                      
            [parameter(Mandatory=$true)]  [string] $AdminArea,            
            [parameter(Mandatory=$true)]  [string] $ScriptPfad
        )
        
        # Vorbereitung
            Get-Variable -Scope global | Where-Object {$_.name -eq 'LogFilePfad'} | Remove-Variable -Scope global
            New-Variable -Name 'LogFilePfad' -Scope global -Value "$ScriptPfad\Runtime.log"
            protokoll -Eintrag $('#' * 80) -ForegroundColor white
            protokoll -Eintrag "konfiguriere AdminArea" -ForegroundColor white
            
        # erstelle OU-Struktur  
            Protokoll -Eintrag "   erstelle Organisationseinheiten" -ForegroundColor white    
            $OUs = $AdminArea-split ',' | Where-Object {$_ -like 'OU=*'}
            $OU = (Get-ADDomain).distinguishedname
            for ($i = $OUs.Count - 1; $i -ge 0; $i = $i - 1) {         
                erstelle_OUs -OUPfad $OU -OUName ($OUs[$i] -replace 'OU=')
                $OU = $OUs[$i] + ',' + $OU
            }   
        # erstelle SubOUs 
            erstelle_OUs -OUName "Benutzer" -OUPfad $OU
            erstelle_OUs -OUName "Gruppen"  -OUPfad $OU
            erstelle_OUs -OUName "System"   -OUPfad $OU
                    
        # erstelle Gruppen
            Protokoll -Eintrag "   erstelle Basisgruppen" -ForegroundColor white
            erstelle-ADGroups -GroupName "GG-NoAccess"       -GroupScope Global      -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription 'Gruppe ohne Rechte - als PrimaryGroup einsetzbar für ZeroPermission!'
            erstelle-ADGroups -GroupName "LD-AD-AdminArea-R" -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription 'Recht Lesen auf AdminArea-OU für GPO-Processing und RechteMapping'
       
        # verschiebe SystemGruppen
            Protokoll -Eintrag "   verschiebe SystemGruppen nach 'OU=System,$AdminArea'" -ForegroundColor white
            Protokoll -Eintrag "      'Protected Users'" -ForegroundColor green
            Move-ADObject -Identity (Get-ADGroup 'Protected Users') -TargetPath "OU=System,$AdminArea"

            Protokoll -Eintrag "      'Domain Admins'" -ForegroundColor green
            Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-512'} | Move-ADObject -TargetPath "OU=System,$AdminArea"

            Protokoll -Eintrag "      'Schema-Admins'" -ForegroundColor green
            Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-518'} | Move-ADObject -TargetPath "OU=System,$AdminArea"

            Protokoll -Eintrag "      'Organisation Admins'" -ForegroundColor green
            Get-ADGroup -Filter 'Admincount -eq 1' | Where-Object {$_.SID -like '*-519'} | Move-ADObject -TargetPath "OU=System,$AdminArea"

        # Anpassung Administrator (500)
            $AdminUser = (Get-ADUser -Filter 'Admincount -eq 1'| Where-Object {$_.SID -like '*-500'}).SamaccountName

            Protokoll -Eintrag "   passe den Account $AdminUser an" -ForegroundColor white
            Protokoll -Eintrag "      verschiebe $AdminUser nach 'OU=Benutzer,$AdminArea'" -ForegroundColor green
            
            Get-ADUser -Identity $AdminUser | Move-ADObject -TargetPath "OU=Benutzer,$AdminArea"
                        
            neues-ADGroupMember -GroupName 'Protected Users' -MemberName $AdminUser
            
        # sammle zu bearbeitende OUs
            Protokoll -Eintrag "ACHTUNG: Die AdminArea kann durch Entfernen von Standardrechten" -ForegroundColor yellow
            Protokoll -Eintrag "         geschuetzt werden. Sollte das nicht gewuenscht sein,  " -ForegroundColor yellow
            Protokoll -Eintrag "         dann druecke jetzt STRG+C. Weiter gehts mit ENTER.    " -ForegroundColor yellow
            pause
            Protokoll -Eintrag "   konfiguriere ACL der AdminArea" -ForegroundColor white            
            $OUs = @()
            $OUs += $AdminArea
            $OUs += "OU=Benutzer,$AdminArea"
            $OUs += "OU=Gruppen,$AdminArea"
            $OUs += "OU=System,$AdminArea"

        # editiere ACL für jede OU
            $OUs | ForEach-Object {
                $OU = $_
                Protokoll -Eintrag "      $OU" -ForegroundColor green

                # lese aktuelle ACL
                    $ADO = [ADSI]"LDAP://$OU"
                    $ACL = $ADO.psbase.ObjectSecurity  

                # deaktiviere Vererbung der Delegation
                    $ACL.SetAccessRuleProtection($true,$false)

                # suche und entferne ACE 
                    # für Authenticated Users
                        $IdentAuthUser = Get-ADAccountAuthorizationGroup -Identity ($env:USERNAME) | 
                            Where-Object {$_.SID -eq 'S-1-5-11'} |
                                Select-Object -ExpandProperty name

                        $ACEs = $ACL.Access | Where-Object {$_.identityreference -like "NT*\$IdentAuthUser"}
                        $ACEs | ForEach-Object {
                            $ACL.RemoveAccessRule($_) | Out-Null 
                        }

                    # für Druck Operatoren
                        $IdentAuthUser = Get-ADGroup -Filter 'Admincount -eq 1' | 
                            Where-Object {$_.SID -like '*-550'} | 
                                Select-Object -ExpandProperty SamAccountName

                        $ACEs = $ACL.Access | Where-Object {$_.identityreference -like "*\$IdentAuthUser"}
                        $ACEs | ForEach-Object {
                            $ACL.RemoveAccessRule($_) | Out-Null 
                        }

                    # für Konten-Operatoren
                        $IdentAuthUser = Get-ADGroup -Filter 'Admincount -eq 1' | 
                            Where-Object {$_.SID -like '*-548'} | 
                                Select-Object -ExpandProperty SamAccountName

                        $ACEs = $ACL.Access | Where-Object {$_.identityreference -like "*\$IdentAuthUser"}
                        $ACEs | ForEach-Object {
                            $ACL.RemoveAccessRule($_) | Out-Null 
                        }

                # schreibe weitere ACEs
                    # Organisations-Admins == GenericAll
                        $IdentityAD = Get-ADGroup -Filter 'Admincount -eq 1' | 
                            Where-Object {$_.SID -like '*-519'} | 
                                Select-Object -ExpandProperty SamAccountName

                        $Action     = [System.Security.AccessControl.AccessControlType]::Allow
                        $Permission = [System.DirectoryServices.ActiveDirectoryRights]::GenericAll  
                        $Identity   = New-Object -TypeName System.Security.Principal.NTAccount -ArgumentList "", "$IdentityAD"  

                        $ACE = New-Object -TypeName System.DirectoryServices.ActiveDirectoryAccessRule -ArgumentList $Identity, $Permission, $Action
                        $ACL.AddAccessRule($ACE)

                    # Administratoren == GenericAll
                        $IdentityAD = Get-ADGroup -Filter 'Admincount -eq 1' | 
                            Where-Object {$_.SID -like '*-544'} | 
                                Select-Object -ExpandProperty SamAccountName

                        $Action     = [System.Security.AccessControl.AccessControlType]::Allow
                        $Permission = [System.DirectoryServices.ActiveDirectoryRights]::GenericAll  
                        $Identity   = New-Object -TypeName System.Security.Principal.NTAccount -ArgumentList "", "$IdentityAD"  

                        $ACE = New-Object -TypeName System.DirectoryServices.ActiveDirectoryAccessRule -ArgumentList $Identity, $Permission, $Action
                        $ACL.AddAccessRule($ACE)

                    # LD-AD-AdminArea-R == Auflisten aller AdminArea-OUs                       
                        $IdentityAD = Get-ADGroup 'LD-AD-AdminArea-R' | Select-Object -ExpandProperty SamAccountName

                        $Action     = [System.Security.AccessControl.AccessControlType]::Allow                        
                        $Identity   = New-Object -TypeName System.Security.Principal.NTAccount -ArgumentList "", "$IdentityAD"  

                        $Permission = [System.DirectoryServices.ActiveDirectoryRights]::ReadProperty  
                        $ACE = New-Object -TypeName System.DirectoryServices.ActiveDirectoryAccessRule -ArgumentList $Identity, $Permission, $Action
                        $ACL.AddAccessRule($ACE)

                    # LD-AD-AdminArea-R == ListChilds
                        $Identity   = New-Object -TypeName System.Security.Principal.NTAccount -ArgumentList "", "LD-AD-AdminArea-R"  

                        $Action     = [System.Security.AccessControl.AccessControlType]::Allow 
                        $Permission = [System.DirectoryServices.ActiveDirectoryRights]::ListChildren  
                            
                        $ACE = New-Object -TypeName System.DirectoryServices.ActiveDirectoryAccessRule -ArgumentList $Identity, $Permission, $Action
                        $ACL.AddAccessRule($ACE)
                                            
                    # DomainComputer == Lesen auf OU=Benutzer (GPO)
                        if ($OU -like 'OU=Benutzer,*') {
                            $IdentityAD = Get-ADGroup -Filter '*' | 
                                Where-Object {$_.SID -like '*-515'} | 
                                    Select-Object -ExpandProperty SamAccountName

                            $Action     = [System.Security.AccessControl.AccessControlType]::Allow
                            $Permission = [System.DirectoryServices.ActiveDirectoryRights]::GenericRead  
                            $Identity   = New-Object -TypeName System.Security.Principal.NTAccount -ArgumentList "", "$IdentityAD"  

                            $ACE = New-Object -TypeName System.DirectoryServices.ActiveDirectoryAccessRule -ArgumentList $Identity, $Permission, $Action
                            $ACL.AddAccessRule($ACE)
                        }
    
                # schreibe ACL zurück    
                    $ADO.psbase.CommitChanges() 
            }
    }

    function erstelle-SecurityScope {
        param(
            [parameter(Mandatory=$true)]  [string] $ScopeName,            
            [parameter(Mandatory=$true)]  [string] $AdminArea,
            [parameter(Mandatory=$true)]  [string] $ScopeOU,
            [parameter(Mandatory=$true)]  [string] $ScriptPfad,
            [parameter(Mandatory=$false)] [string] $PrefixGPO = "GPO-Computer-Sicherheit",
            [parameter(Mandatory=$false)] [string] $PrefixGG  = "GG-SEC",
            [parameter(Mandatory=$false)] [string] $PrefixLD  = "LD-SEC",
            [parameter(Mandatory=$false)] [switch] $OhneProtectedUser = $false
        )
        
        # Vorbereitung
            Get-Variable -Scope global | Where-Object {$_.name -eq 'LogFilePfad'} | Remove-Variable -Scope global
            New-Variable -Name 'LogFilePfad' -Scope global -Value "$ScriptPfad\Runtime.log"
            protokoll -Eintrag $('#' * 80) -ForegroundColor white
            protokoll -Eintrag "erstelle SecurityScope $ScopeName" -ForegroundColor white
            
        # erstelle AD-Gruppen und definiere Gruppenmitgliedschaften
            Protokoll -Eintrag "konfiguriere Gruppen im AD" -ForegroundColor white
            Protokoll -Eintrag "   Rollengruppen" -ForegroundColor white
            erstelle-ADGroups -GroupName "$PrefixGG-$ScopeName-Admins"  -GroupScope Global -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Rolle Administratoren für $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixGG-$ScopeName-Login"   -GroupScope Global -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Rolle Konsolenlogin auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixGG-$ScopeName-RDP"     -GroupScope Global -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Rolle RDP-Login auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixGG-$ScopeName-Task"    -GroupScope Global -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Rolle 'Logon as a Batch Job' auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixGG-$ScopeName-Service" -GroupScope Global -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Rolle 'Logon as a Service' auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixGG-$ScopeName-WinRM"   -GroupScope Global -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Rolle 'Remoteverwaltungsbenutzer (WinRM)' auf $ScopeName" 

            Protokoll -Eintrag "   Rechtegruppen" -ForegroundColor white
            erstelle-ADGroups -GroupName "$PrefixLD-$ScopeName-Admins"  -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Recht Administratoren für $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixLD-$ScopeName-Login"   -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Recht Konsolenlogin auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixLD-$ScopeName-RDP"     -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Recht RDP-Login auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixLD-$ScopeName-Task"    -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Recht 'Logon as a Batch Job' auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixLD-$ScopeName-Service" -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Recht 'Logon as a Service' auf $ScopeName" 
            erstelle-ADGroups -GroupName "$PrefixLD-$ScopeName-WinRM"   -GroupScope DomainLocal -GroupPath "OU=Gruppen,$AdminArea" -GroupDescription "Recht 'Remoteverwaltungsbenutzer (WinRM)' auf $ScopeName" 

            Protokoll -Eintrag "   definiere Gruppenmitgliedschaften" -ForegroundColor white
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-Admins"  -MemberName "$PrefixGG-$ScopeName-Admins"            
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-Login"   -MemberName "$PrefixGG-$ScopeName-Admins"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-RDP"     -MemberName "$PrefixGG-$ScopeName-Admins"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-WinRM"   -MemberName "$PrefixGG-$ScopeName-Admins"
                        
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-Login"   -MemberName "$PrefixGG-$ScopeName-Login"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-Login"   -MemberName "$PrefixGG-$ScopeName-RDP"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-RDP"     -MemberName "$PrefixGG-$ScopeName-RDP"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-Task"    -MemberName "$PrefixGG-$ScopeName-Task"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-Service" -MemberName "$PrefixGG-$ScopeName-Service"
            neues-ADGroupMember -GroupName "$PrefixLD-$ScopeName-WinRM"   -MemberName "$PrefixGG-$ScopeName-WinRM"

            if ($OhneProtectedUser -eq $false) {
                neues-ADGroupMember -GroupName "Protected Users"          -MemberName "$PrefixGG-$ScopeName-Admins"                      
            }
            neues-ADGroupMember -GroupName "LD-AD-AdminArea-R"            -MemberName "$PrefixGG-$ScopeName-Admins"
                   
        # erstelle Management-OU
            protokoll -Eintrag "erstelle Management-OU" -ForegroundColor white
            erstelle_OUs -OUName $ScopeOU -OUDescription "Computer mit SecurityScope $ScopeName" -ProtOffset '   '
    
        # erstelle GPO
            Protokoll -Eintrag "konfiguriere GPO '$PrefixGPO-Scope-$ScopeName'" -ForegroundColor white

            $GPO = Get-GPO -All | Where-Object {$_.DisplayName -eq "$PrefixGPO-Scope-$ScopeName"}
            if ($GPO) {
                Protokoll -Eintrag "   überschreibe die GPO '$PrefixGPO-Scope-$ScopeName'" -ForegroundColor yellow  
            } else {
                Protokoll -Eintrag "   erstelle leere GPO" -ForegroundColor white
                New-GPO -Name "$PrefixGPO-Scope-$ScopeName" | Out-Null
            }

            Protokoll -Eintrag "   kopiere Template" -ForegroundColor white
            Copy-Item -Path "$ScriptPfad\GPOTemplate" -Destination "$ScriptPfad\GPO-$ScopeName" -Recurse

            Protokoll -Eintrag "   editiere Template" -ForegroundColor white
            $GPOInf = get-content -Path (Get-ChildItem -Path "$ScriptPfad\GPO-$ScopeName" -Recurse -Filter *.inf).FullName
            $GPOInf | ForEach-Object {
                    $Zeile = $_
                    # Gruppenmitgliedschaft Administratoren
                        if ($Zeile -like '`*S-1-5-32-544__Members*') {
                            $Zeile = "*S-1-5-32-544__Members = *" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-Admins")
                        }
                    # Gruppenmitgliedschaft RemoteDesktopBenutzer
                        if ($Zeile -like '`*S-1-5-32-555__Members*') {
                            $Zeile = "*S-1-5-32-555__Members = *" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-RDP")
                        }    
                    # Gruppenmitgliedschaft Remoteverwaltungsbenutzer
                        if ($Zeile -like '`*S-1-5-32-580__Members*') {
                            $Zeile = "*S-1-5-32-580__Members = *" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-WinRM")
                        }                
                    # Recht interactiveLogon
                        if ($Zeile -like 'SeInteractiveLogonRight*') {
                            $Zeile = "SeInteractiveLogonRight = *S-1-5-32-544,*" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-Login")
                        }
                    # Recht RDP
                        if ($Zeile -like 'SeRemoteInteractiveLogonRight*') {
                            $Zeile = "SeRemoteInteractiveLogonRight = *" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-RDP") 
                        }
                    # Recht 'Logon as a Batch Job'
                        if ($Zeile -like 'SeBatchLogonRight*') {
                            $Zeile = "SeBatchLogonRight = *" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-Task")
                        }
                    # Recht 'Logon as a Service'
                        if ($Zeile -like 'SeServiceLogonRight*') {
                            $Zeile = "SeServiceLogonRight = *S-1-5-80-0,*S-1-5-83-0,*" + (ermittle-SID -ADObject "$PrefixLD-$ScopeName-Service")
                        }

                    $Zeile
                } | Out-File -FilePath (Get-ChildItem -Path "$ScriptPfad\GPO-$ScopeName" -Recurse -Filter *.inf).FullName
                
            Protokoll -Eintrag "   importiere Template in die GPO" -ForegroundColor white
            Import-GPO -TargetName "$PrefixGPO-Scope-$ScopeName" -Path "$ScriptPfad\GPO-$ScopeName" -BackupId '57AB08E6-7EFF-4131-9C12-39A7CCF6657B' | Out-Null

            Protokoll -Eintrag "   entferne Template" -ForegroundColor white
            Remove-Item -Path "$ScriptPfad\GPO-$ScopeName" -Recurse -Force

            if (-not $GPO) {                
                Protokoll -Eintrag "   linke GPO auf ManagementOU" -ForegroundColor white
                $GUID = (Get-GPO -Name "$PrefixGPO-Scope-$ScopeName").ID.GUID
                New-GPLink -Guid $GUID -Target "$ScopeOU" -ErrorAction SilentlyContinue | Out-Null
            }            
    }

    function erstelle-AdminUser {
        param(
            [parameter(Mandatory=$true)]  [string] $ScopeName,
            [parameter(Mandatory=$true)]  [string] $AdminUser,
            [parameter(Mandatory=$true)]  [string] $StandardUser,
            [parameter(Mandatory=$true)]  [string] $AdminArea,            
            [parameter(Mandatory=$true)]  [string] $ScriptPfad,            
            [parameter(Mandatory=$false)] [string] $PrefixGG  = "GG-SEC",
            [parameter(Mandatory=$false)] [string] $PrefixLD  = "LD-SEC"
        )
        
        # Vorbereitung
            Get-Variable -Scope global | Where-Object {$_.name -eq 'LogFilePfad'} | Remove-Variable -Scope global
            New-Variable -Name 'LogFilePfad' -Scope global -Value "$ScriptPfad\Runtime.log"
            protokoll -Eintrag $('#' * 80) -ForegroundColor white
            protokoll -Eintrag "erstelle AdminBenutzer $AdminUser" -ForegroundColor white

        # erstelle AdminAccount            
            erstelle-ADUser -UserName $AdminUser -UserPath "OU=Benutzer,$AdminArea" -UserDescription "AdminUser für $StandardUser"

            Protokoll -Eintrag "   definiere Gruppenmitgliedschaften" -ForegroundColor White
            neues-ADGroupMember -GroupName "$PrefixGG-$ScopeName-Admins" -MemberName $AdminUser
            neues-ADGroupMember -GroupName "GG-NoAccess"                 -MemberName $AdminUser
    
            Protokoll -Eintrag "   definiere PrimaryGroup == GG-NoAccess" -ForegroundColor White
            $SID  = ((Get-ADGroup -Identity "GG-NoAccess").SID.value -split '-')[-1]
            Set-ADUser -Identity $AdminUser -Replace @{"primaryGroupID"=$SID}

            $SID = (Get-ADDomain).DomainSID.value + '-513'    
            Remove-ADGroupMember -Identity (Get-ADGroup -filter "SID -eq '$SID'") -Members $AdminUser -Confirm:$false        
    }

# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUPyfVoo1J++brfVkIPOQR50rx
# 4a6gggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBSOesXahN/BrCAcedv+MHtONjwnDTANBgkqhkiG9w0BAQEF
# AASCAQBbb4/M4pK5Hq9Lj6PSXuI9wWr769f7yUcAZXuV+FZtDbwR3gCsKwOqHdFq
# cPyPYwcImOlo6HzaWeZx9mNcfdQWlkbe/2qTynMHkr6030iTHITNiTtgmffUB0he
# WCYyB/MyWhrU1tAOomqOlLMH/C35thYslUqE3en8PgBfgnfCHJysTup+Q/EiVAE7
# QoZRBllJopOcpMXlzn32e0FjoizN21Z+cCGEQC4bfFOKF0XLBVj2V6sKsw0oH5gN
# nnVKhRD+TCKguK+wfH9E4ZWJ8ypnIJbtvB3+WPhoGYYJB3Z8odyeBqLm2qxN88MA
# q3VgZFeqvYexKWiUSGLUCvpunb/V
# SIG # End signature block
